from . import surfacemeshing

def find_triangle_geom_centroids(data_container_array, feature_attribute_matrix_name = None, face_labels_array_path = None, centroids_array_name = None, observer = None) -> int:
  """
  FindTriangleGeomCentroids

  :param data_container_array | DataContainerArray
  :param feature_attribute_matrix_name | DataArrayPath FeatureAttributeMatrixName
  :param face_labels_array_path | DataArrayPath FaceLabelsArrayPath
  :param centroids_array_name | QString CentroidsArrayName
  :param observer | Observer
  """

  filter = surfacemeshing.FindTriangleGeomCentroids()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if feature_attribute_matrix_name is not None:
    filter.FeatureAttributeMatrixName = feature_attribute_matrix_name
  if face_labels_array_path is not None:
    filter.FaceLabelsArrayPath = face_labels_array_path
  if centroids_array_name is not None:
    filter.CentroidsArrayName = centroids_array_name
  filter.execute()
  return filter.ErrorCode

def find_triangle_geom_neighbors(data_container_array, feature_attribute_matrix_path = None, neighbor_list_array_name = None, face_labels_array_path = None, num_neighbors_array_name = None, observer = None) -> int:
  """
  FindTriangleGeomNeighbors

  :param data_container_array | DataContainerArray
  :param feature_attribute_matrix_path | DataArrayPath FeatureAttributeMatrixPath
  :param neighbor_list_array_name | QString NeighborListArrayName
  :param face_labels_array_path | DataArrayPath FaceLabelsArrayPath
  :param num_neighbors_array_name | QString NumNeighborsArrayName
  :param observer | Observer
  """

  filter = surfacemeshing.FindTriangleGeomNeighbors()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if feature_attribute_matrix_path is not None:
    filter.FeatureAttributeMatrixPath = feature_attribute_matrix_path
  if neighbor_list_array_name is not None:
    filter.NeighborListArrayName = neighbor_list_array_name
  if face_labels_array_path is not None:
    filter.FaceLabelsArrayPath = face_labels_array_path
  if num_neighbors_array_name is not None:
    filter.NumNeighborsArrayName = num_neighbors_array_name
  filter.execute()
  return filter.ErrorCode

def find_triangle_geom_shapes(data_container_array, feature_attribute_matrix_name = None, face_labels_array_path = None, centroids_array_path = None, volumes_array_path = None, omega3s_array_name = None, axis_lengths_array_name = None, axis_euler_angles_array_name = None, aspect_ratios_array_name = None, observer = None) -> int:
  """
  FindTriangleGeomShapes

  :param data_container_array | DataContainerArray
  :param feature_attribute_matrix_name | DataArrayPath FeatureAttributeMatrixName
  :param face_labels_array_path | DataArrayPath FaceLabelsArrayPath
  :param centroids_array_path | DataArrayPath CentroidsArrayPath
  :param volumes_array_path | DataArrayPath VolumesArrayPath
  :param omega3s_array_name | QString Omega3sArrayName
  :param axis_lengths_array_name | QString AxisLengthsArrayName
  :param axis_euler_angles_array_name | QString AxisEulerAnglesArrayName
  :param aspect_ratios_array_name | QString AspectRatiosArrayName
  :param observer | Observer
  """

  filter = surfacemeshing.FindTriangleGeomShapes()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if feature_attribute_matrix_name is not None:
    filter.FeatureAttributeMatrixName = feature_attribute_matrix_name
  if face_labels_array_path is not None:
    filter.FaceLabelsArrayPath = face_labels_array_path
  if centroids_array_path is not None:
    filter.CentroidsArrayPath = centroids_array_path
  if volumes_array_path is not None:
    filter.VolumesArrayPath = volumes_array_path
  if omega3s_array_name is not None:
    filter.Omega3sArrayName = omega3s_array_name
  if axis_lengths_array_name is not None:
    filter.AxisLengthsArrayName = axis_lengths_array_name
  if axis_euler_angles_array_name is not None:
    filter.AxisEulerAnglesArrayName = axis_euler_angles_array_name
  if aspect_ratios_array_name is not None:
    filter.AspectRatiosArrayName = aspect_ratios_array_name
  filter.execute()
  return filter.ErrorCode

def find_triangle_geom_sizes(data_container_array, face_labels_array_path = None, feature_attribute_matrix_name = None, volumes_array_name = None, observer = None) -> int:
  """
  FindTriangleGeomSizes

  :param data_container_array | DataContainerArray
  :param face_labels_array_path | DataArrayPath FaceLabelsArrayPath
  :param feature_attribute_matrix_name | DataArrayPath FeatureAttributeMatrixName
  :param volumes_array_name | QString VolumesArrayName
  :param observer | Observer
  """

  filter = surfacemeshing.FindTriangleGeomSizes()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if face_labels_array_path is not None:
    filter.FaceLabelsArrayPath = face_labels_array_path
  if feature_attribute_matrix_name is not None:
    filter.FeatureAttributeMatrixName = feature_attribute_matrix_name
  if volumes_array_name is not None:
    filter.VolumesArrayName = volumes_array_name
  filter.execute()
  return filter.ErrorCode

def laplacian_smoothing(data_container_array, surface_data_container_name = None, surface_mesh_node_type_array_path = None, surface_mesh_face_labels_array_path = None, iteration_steps = None, lambda_value = None, surface_point_lambda = None, triple_line_lambda = None, quad_point_lambda = None, surface_triple_line_lambda = None, surface_quad_point_lambda = None, use_taubin_smoothing = None, mu_factor = None, observer = None) -> int:
  """
  LaplacianSmoothing

  :param data_container_array | DataContainerArray
  :param surface_data_container_name | DataArrayPath SurfaceDataContainerName
  :param surface_mesh_node_type_array_path | DataArrayPath SurfaceMeshNodeTypeArrayPath
  :param surface_mesh_face_labels_array_path | DataArrayPath SurfaceMeshFaceLabelsArrayPath
  :param iteration_steps | int IterationSteps
  :param lambda_value | float Lambda
  :param surface_point_lambda | float SurfacePointLambda
  :param triple_line_lambda | float TripleLineLambda
  :param quad_point_lambda | float QuadPointLambda
  :param surface_triple_line_lambda | float SurfaceTripleLineLambda
  :param surface_quad_point_lambda | float SurfaceQuadPointLambda
  :param use_taubin_smoothing | bool UseTaubinSmoothing
  :param mu_factor | float MuFactor
  :param observer | Observer
  """

  filter = surfacemeshing.LaplacianSmoothing()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if surface_data_container_name is not None:
    filter.SurfaceDataContainerName = surface_data_container_name
  if surface_mesh_node_type_array_path is not None:
    filter.SurfaceMeshNodeTypeArrayPath = surface_mesh_node_type_array_path
  if surface_mesh_face_labels_array_path is not None:
    filter.SurfaceMeshFaceLabelsArrayPath = surface_mesh_face_labels_array_path
  if iteration_steps is not None:
    filter.IterationSteps = iteration_steps
  if lambda_value is not None:
    filter.Lambda = lambda_value
  if surface_point_lambda is not None:
    filter.SurfacePointLambda = surface_point_lambda
  if triple_line_lambda is not None:
    filter.TripleLineLambda = triple_line_lambda
  if quad_point_lambda is not None:
    filter.QuadPointLambda = quad_point_lambda
  if surface_triple_line_lambda is not None:
    filter.SurfaceTripleLineLambda = surface_triple_line_lambda
  if surface_quad_point_lambda is not None:
    filter.SurfaceQuadPointLambda = surface_quad_point_lambda
  if use_taubin_smoothing is not None:
    filter.UseTaubinSmoothing = use_taubin_smoothing
  if mu_factor is not None:
    filter.MuFactor = mu_factor
  filter.execute()
  return filter.ErrorCode

def quick_surface_mesh(data_container_array, selected_data_array_paths = None, surface_data_container_name = None, vertex_attribute_matrix_name = None, face_attribute_matrix_name = None, feature_ids_array_path = None, face_labels_array_name = None, node_types_array_name = None, feature_attribute_matrix_name = None, observer = None) -> int:
  """
  QuickSurfaceMesh

  :param data_container_array | DataContainerArray
  :param selected_data_array_paths | std::vector<DataArrayPath> SelectedDataArrayPaths
  :param surface_data_container_name | DataArrayPath SurfaceDataContainerName
  :param vertex_attribute_matrix_name | QString VertexAttributeMatrixName
  :param face_attribute_matrix_name | QString FaceAttributeMatrixName
  :param feature_ids_array_path | DataArrayPath FeatureIdsArrayPath
  :param face_labels_array_name | QString FaceLabelsArrayName
  :param node_types_array_name | QString NodeTypesArrayName
  :param feature_attribute_matrix_name | QString FeatureAttributeMatrixName
  :param observer | Observer
  """

  filter = surfacemeshing.QuickSurfaceMesh()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if selected_data_array_paths is not None:
    filter.SelectedDataArrayPaths = selected_data_array_paths
  if surface_data_container_name is not None:
    filter.SurfaceDataContainerName = surface_data_container_name
  if vertex_attribute_matrix_name is not None:
    filter.VertexAttributeMatrixName = vertex_attribute_matrix_name
  if face_attribute_matrix_name is not None:
    filter.FaceAttributeMatrixName = face_attribute_matrix_name
  if feature_ids_array_path is not None:
    filter.FeatureIdsArrayPath = feature_ids_array_path
  if face_labels_array_name is not None:
    filter.FaceLabelsArrayName = face_labels_array_name
  if node_types_array_name is not None:
    filter.NodeTypesArrayName = node_types_array_name
  if feature_attribute_matrix_name is not None:
    filter.FeatureAttributeMatrixName = feature_attribute_matrix_name
  filter.execute()
  return filter.ErrorCode

def reverse_triangle_winding(data_container_array, surface_data_container_name = None, observer = None) -> int:
  """
  ReverseTriangleWinding

  :param data_container_array | DataContainerArray
  :param surface_data_container_name | DataArrayPath SurfaceDataContainerName
  :param observer | Observer
  """

  filter = surfacemeshing.ReverseTriangleWinding()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if surface_data_container_name is not None:
    filter.SurfaceDataContainerName = surface_data_container_name
  filter.execute()
  return filter.ErrorCode

def shared_feature_face_filter(data_container_array, face_feature_attribute_matrix_name = None, surface_mesh_feature_face_ids_array_name = None, surface_mesh_feature_face_labels_array_name = None, surface_mesh_feature_face_num_triangles_array_name = None, surface_mesh_face_labels_array_path = None, observer = None) -> int:
  """
  SharedFeatureFaceFilter

  :param data_container_array | DataContainerArray
  :param face_feature_attribute_matrix_name | QString FaceFeatureAttributeMatrixName
  :param surface_mesh_feature_face_ids_array_name | QString SurfaceMeshFeatureFaceIdsArrayName
  :param surface_mesh_feature_face_labels_array_name | QString SurfaceMeshFeatureFaceLabelsArrayName
  :param surface_mesh_feature_face_num_triangles_array_name | QString SurfaceMeshFeatureFaceNumTrianglesArrayName
  :param surface_mesh_face_labels_array_path | DataArrayPath SurfaceMeshFaceLabelsArrayPath
  :param observer | Observer
  """

  filter = surfacemeshing.SharedFeatureFaceFilter()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if face_feature_attribute_matrix_name is not None:
    filter.FaceFeatureAttributeMatrixName = face_feature_attribute_matrix_name
  if surface_mesh_feature_face_ids_array_name is not None:
    filter.SurfaceMeshFeatureFaceIdsArrayName = surface_mesh_feature_face_ids_array_name
  if surface_mesh_feature_face_labels_array_name is not None:
    filter.SurfaceMeshFeatureFaceLabelsArrayName = surface_mesh_feature_face_labels_array_name
  if surface_mesh_feature_face_num_triangles_array_name is not None:
    filter.SurfaceMeshFeatureFaceNumTrianglesArrayName = surface_mesh_feature_face_num_triangles_array_name
  if surface_mesh_face_labels_array_path is not None:
    filter.SurfaceMeshFaceLabelsArrayPath = surface_mesh_face_labels_array_path
  filter.execute()
  return filter.ErrorCode

def triangle_area_filter(data_container_array, surface_mesh_triangle_areas_array_path = None, observer = None) -> int:
  """
  TriangleAreaFilter

  :param data_container_array | DataContainerArray
  :param surface_mesh_triangle_areas_array_path | DataArrayPath SurfaceMeshTriangleAreasArrayPath
  :param observer | Observer
  """

  filter = surfacemeshing.TriangleAreaFilter()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if surface_mesh_triangle_areas_array_path is not None:
    filter.SurfaceMeshTriangleAreasArrayPath = surface_mesh_triangle_areas_array_path
  filter.execute()
  return filter.ErrorCode

def triangle_centroid_filter(data_container_array, surface_mesh_triangle_centroids_array_path = None, observer = None) -> int:
  """
  TriangleCentroidFilter

  :param data_container_array | DataContainerArray
  :param surface_mesh_triangle_centroids_array_path | DataArrayPath SurfaceMeshTriangleCentroidsArrayPath
  :param observer | Observer
  """

  filter = surfacemeshing.TriangleCentroidFilter()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if surface_mesh_triangle_centroids_array_path is not None:
    filter.SurfaceMeshTriangleCentroidsArrayPath = surface_mesh_triangle_centroids_array_path
  filter.execute()
  return filter.ErrorCode

def triangle_dihedral_angle_filter(data_container_array, surface_mesh_triangle_dihedral_angles_array_path = None, observer = None) -> int:
  """
  TriangleDihedralAngleFilter

  :param data_container_array | DataContainerArray
  :param surface_mesh_triangle_dihedral_angles_array_path | DataArrayPath SurfaceMeshTriangleDihedralAnglesArrayPath
  :param observer | Observer
  """

  filter = surfacemeshing.TriangleDihedralAngleFilter()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if surface_mesh_triangle_dihedral_angles_array_path is not None:
    filter.SurfaceMeshTriangleDihedralAnglesArrayPath = surface_mesh_triangle_dihedral_angles_array_path
  filter.execute()
  return filter.ErrorCode

def triangle_normal_filter(data_container_array, surface_mesh_triangle_normals_array_path = None, observer = None) -> int:
  """
  TriangleNormalFilter

  :param data_container_array | DataContainerArray
  :param surface_mesh_triangle_normals_array_path | DataArrayPath SurfaceMeshTriangleNormalsArrayPath
  :param observer | Observer
  """

  filter = surfacemeshing.TriangleNormalFilter()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if surface_mesh_triangle_normals_array_path is not None:
    filter.SurfaceMeshTriangleNormalsArrayPath = surface_mesh_triangle_normals_array_path
  filter.execute()
  return filter.ErrorCode

def generate_geometry_connectivity(data_container_array, surface_data_container_name = None, generate_vertex_triangle_lists = None, generate_triangle_neighbors = None, observer = None) -> int:
  """
  GenerateGeometryConnectivity

  :param data_container_array | DataContainerArray
  :param surface_data_container_name | DataArrayPath SurfaceDataContainerName
  :param generate_vertex_triangle_lists | bool GenerateVertexTriangleLists
  :param generate_triangle_neighbors | bool GenerateTriangleNeighbors
  :param observer | Observer
  """

  filter = surfacemeshing.GenerateGeometryConnectivity()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if surface_data_container_name is not None:
    filter.SurfaceDataContainerName = surface_data_container_name
  if generate_vertex_triangle_lists is not None:
    filter.GenerateVertexTriangleLists = generate_vertex_triangle_lists
  if generate_triangle_neighbors is not None:
    filter.GenerateTriangleNeighbors = generate_triangle_neighbors
  filter.execute()
  return filter.ErrorCode

