# DREAM3D and Python (Anaconda Python) #

In order to use DREAM3D from within a Python environment you will need to pick one of two options:

+ 1: Install a prebuilt 'dream3d-conda' package
+ 2: Build DREAM.3D from source using the ananconda/condaforge environment


## Install a Prebuilt 'dream3d-conda' Package ##

See the instructions in the file python\_filter\_development.md

### Writing a DREAM3D Filter in Python ###

Please see the file _conda\_filter\_development.md_ file for instructions.

## Building a DREAM3D Anaconda Package ##

### Requirements ###

+ Install [Visual Studio](https://visualstudio.microsoft.com/downloads/) (Windows)
+ Install [Anaconda](https://docs.anaconda.com/anaconda/install/)
+ Setup [conda-forge](https://conda-forge.org/) channel (requires conda >= 4.6)

### Clone the DREAM3D git repositories ###

You wil need to clone DREAM3D development repos into a new directory of your choice

```lang-console
(base) C:\Users\johnsmith> mkdir DREAM3D-Dev
(base) C:\Users\johnsmith> cd DREAM3D-Dev
(base) C:\Users\johnsmith\DREAM3D-Dev> git clone https://github.com/BlueQuartzSoftware/DREAM3D
(base) C:\Users\johnsmith\DREAM3D-Dev> git clone https://github.com/BlueQuartzSoftware/SIMPL
(base) C:\Users\johnsmith\DREAM3D-Dev> git clone https://github.com/BlueQuartzSoftware/SIMPLView
(base) C:\Users\johnsmith\DREAM3D-Dev> git clone https://github.com/BlueQuartzSoftware/EbsdLib
(base) C:\Users\johnsmith\DREAM3D-Dev> git clone https://github.com/BlueQuartzSoftware/H5Support
(base) C:\Users\johnsmith\DREAM3D-Dev> git clone https://github.com/BlueQuartzSoftware/CMP
(base) C:\Users\johnsmith\DREAM3D-Dev> git clone https://github.com/dream3d/DREAM3D_Data
```

We now need to clone any additional plugins that are needed. The below are the 4 standard plugins that are
used in DREAM.3D builds.
```lang-console
(base) C:\Users\johnsmith\DREAM3D-Dev> mkdir DREAM3D_Plugins
(base) C:\Users\johnsmith\DREAM3D-Dev> cd DREAM3D_Plugins
(base) C:\Users\johnsmith\DREAM3D-Dev\DREAM3D_Plugins> git clone https://github.com/BlueQuartzSoftware/SimulationIO
(base) C:\Users\johnsmith\DREAM3D-Dev\DREAM3D_Plugins> git clone https://github.com/BlueQuartzSoftware/ITKImageProcessing
(base) C:\Users\johnsmith\DREAM3D-Dev\DREAM3D_Plugins> git clone https://github.com/dream3d/DREAM3DReview
(base) C:\Users\johnsmith\DREAM3D-Dev\DREAM3D_Plugins> git clone https://github.com/dream3d/UCSBUtilities
```

We need to decompress (Unzip) the following in DREAM3D_Data directory. Use your favorite decompression tool such as **tar** or **7Zip**.

+ DREAM3D_Data/Data/Image.tar.gz
+ DREAM3D_Data/Data/SmallN100.tar.gz
+ DREAM3D_Data/Data/T12-MAI-2010.tar.gz

### Anaconda Virtual Environment Setup ###

We are now ready to start building the DREAM3D Anaconda package.

+ Add the *conda-forge* channel and set it's priority:

```lang-console
(base) C:\Users\johnsmith> conda config --add channels conda-forge
(base) C:\Users\johnsmith> conda config --set channel_priority strict
```

+ Next we need to create an anaconda virtual environment where we will build the package. In this example we are calling our virtual environment **d3d_embed** and we are going to use python version 3.7. Create a new conda virtual environment:

```lang-console
(base) C:\Users\johnsmith> conda create -n d3d_embed python=3.7
(base) C:\Users\johnsmith> conda activate d3d_embed
(d3d_embed) C:\Users\johnsmith> 
```

+ Now that the virtual environment is setup we need to install the **conda-build** tooling:

```lang-console
(d3d_embed) C:\Users\johnsmith> conda install conda-build
```

+ Update the dream3d-conda package version through the meta.yaml and setup.py files

*DREAM3D/Conda/meta.yaml*
```
{% set version = "X.Y.Z" %}
```
Replace the X.Y.Z with the appropriate version number incremented from the last build

*DREAM3D/Conda/setup.py*
```
    version='X.Y.Z',
```
Replace the X.Y.Z with the **SAME** X.Y.Z from the meta.yaml file.

+ Branding the Distribution: If you have a _Branding_ folder that you would normally use for DREAM3D then you can use this same folder so that the DREAM3D.exe executable will have the proper visual themes. You will need to edit 2 files to ensure that the branding is built into the distribution. Inside of DREAM3D/Conda/bld.bat and build.sh you will need to add the additional argument to the cmake command:

```
  -D BrandedSIMPLView_DIR:PATH=%SRC_DIR%/BrandedDREAM3D ^
```
to the top of the argument list. The same for the build.sh file for unix/linux/macOS environments. Once all of those files are how you want them, then you can move on to actually building the conda package.

+ Use conda build DREAM3D within the **d3d_embded** virtual environment:

The build takes around 15 min to complete on a 12 core machine with SSD storage.

The initial "solving environment" step may take a while in particular, and it does not have a progress indicator.

```lang-console
(d3d_embed) C:\Users\johnsmith> conda build /path/to/DREAM3D/Conda
```

### Installing the new dream3d-conda packge ###

Install locally built *DREAM3D* package into the current virtual environment

+ Make sure the Python version of the environment matches the version used to build DREAM3D.

```lang-console
(d3d_embed) C:\Users\johnsmith> conda install -c local dream3d-conda
```

### Creating Web based package ###

The developer will need to move the package up to a web server directory that can be indexed by `conda index path/to/dir`. Using this command where the target directory is a network mounted disk is not ideal and can cause timeouts. It is better to have a local `conda` folder that you can then do the `conda index [directory]` command and then just copy the entire directory to the web server.

## Running DREAM3D from Anaconda ##

DREAM3D, and related applications, must be run from within the conda environment.

i.e.:

```lang-console
(base) C:\Users\johnsmith> conda activate d3d_embed
(d3d_embed) C:\Users\johnsmith> DREAM3D.exe
(d3d_embed) C:\Users\johnsmith> PipelineRunner --pipeline /path/to/pipeline.json
```

The same environment must also be used for the Python bindings. The Python bindings are within the dream3d package and can be accessed like the following:

```lang-python
from dream3d import simpl
from dream3d import simplpy

dca = simpl.DataContainerArray()
result = simplpy.create_data_container(dca, data_container_name='foo')
```
